#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2018-03-17
#
# Check Point Logserver metrics plugins
# checkpoint_fw_ls / checkpoint_fwm_ls
#
# 2023-04-22: moved metrics file to ~/local/lib/check_mk/gui/plugins/metrics
#

from cmk.gui.i18n import _
from cmk.gui.plugins.metrics.utils import (
    metric_info,
    graph_info,
    perfometer_info
)

metric_info['checkpoint_fw_ls_localloggingwriterate'] = {
    'title': _('Local Logging Writing Rate'),
    'unit': '1/s',
    'color': '11/a',
}
metric_info['checkpoint_fw_ls_logginghandlingrate'] = {
    'title': _('Logging Handling Rate'),
    'unit': '1/s',
    'color': '21/a',
}
metric_info['checkpoint_fw_ls_lsconnsendrate'] = {
    'title': _('Sending Rate'),
    'unit': '1/s',
    'color': '31/a',
}

metric_info['checkpoint_fwm_ls_updatesandlogsindexedrate'] = {
    'title': _('Updates and logs index rate'),
    'unit': '1/s',
    'color': '12/a',
}
metric_info['checkpoint_fwm_ls_updatesandlogsindexedratepeak'] = {
    'title': _('Updates and logs peak index rate'),
    'unit': '1/s',
    'color': '22/a',
}

metric_info['checkpoint_fwm_ls_readlogsrate'] = {
    'title': _('Logs read rate'),
    'unit': '1/s',
    'color': '26/a',
}
metric_info['checkpoint_fwm_ls_readlogsratepeak'] = {
    'title': _('Logs read peak rate'),
    'unit': '1/s',
    'color': '22/a',
}

graph_info['checkpoint_fw_ls_over_all'] = {
    'title': _('Check Point Firewall Logserver: over all'),
    'metrics': [
        ('checkpoint_fw_ls_localloggingwriterate', 'line'),
        ('checkpoint_fw_ls_logginghandlingrate', 'line'),
    ],
    'optional_metrics': [
        'checkpoint_fw_ls_localloggingwriterate',
        'checkpoint_fw_ls_logginghandlingrate',
    ],
    'range': (0, 'checkpoint_fw_ls_logginghandlingrate:max'),
}

graph_info['checkpoint_fw_ls_gateway'] = {
    'title': _('Check Point Firewall Log server'),
    'metrics': [
        ('checkpoint_fw_ls_lsconnsendrate', 'line'),
    ],
    'range': (0, 'checkpoint_fw_ls_lsconnsendrate:max'),
}

graph_info['checkpoint_fwm_ls_management_read'] = {
    'title': _('Check Point Management Firewall Log server read rate'),
    'metrics': [
        ('checkpoint_fwm_ls_readlogsratepeak', 'line'),
        ('checkpoint_fwm_ls_readlogsrate', 'line'),
    ],
    'optional_metrics': [
        'checkpoint_fwm_ls_readlogsratepeak',
        'checkpoint_fwm_ls_readlogsrate',
    ],
    'range': (0, 'checkpoint_fwm_ls_readlogsrate:max'),
}

graph_info['checkpoint_fwm_ls_management_indexed'] = {
    'title': _('Check Point Management Firewall Log server index rate'),
    'metrics': [
        ('checkpoint_fwm_ls_updatesandlogsindexedratepeak', 'line'),
        ('checkpoint_fwm_ls_updatesandlogsindexedrate', 'line'),
    ],
    'optional_metrics': [
        'checkpoint_fwm_ls_updatesandlogsindexedratepeak',
        'checkpoint_fwm_ls_updatesandlogsindexedrate',
    ],
    'range': (0, 'checkpoint_fwm_ls_updatesandlogsindexedrate:max'),
}

perfometer_info.append(('stacked', [
    {
        'type': 'logarithmic',
        'metric': 'checkpoint_fw_ls_localloggingwriterate',
        'half_value': 1000.0,
        'exponent': 2,
    },
    {
        'type': 'logarithmic',
        'metric': 'checkpoint_fw_ls_logginghandlingrate',
        'half_value': 1000.0,
        'exponent': 2,
    },
]))

perfometer_info.append({
    'type': 'logarithmic',
    'metric': 'checkpoint_fw_ls_lsconnsendrate',
    'half_value': 500.0,
    'exponent': 2,
})

perfometer_info.append(('stacked', [
    {
        'type': 'logarithmic',
        'metric': 'checkpoint_fwm_ls_readlogsrate',
        'half_value': 1000.0,
        'exponent': 2,
    },
    {
        'type': 'logarithmic',
        'metric': 'checkpoint_fwm_ls_updatesandlogsindexedrate',
        'half_value': 1000.0,
        'exponent': 2,
    },
]))
