#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2016-07-29
#
# 2023-04-23: moved wato file to ~/local/lib/check_mk/gui/plugins/wato
#

from cmk.gui.i18n import _
from cmk.gui.valuespec import (
    Dictionary,
    TextAscii,
    MonitoringState,
    DropdownChoice,
)
from cmk.gui.plugins.wato.utils import (
    CheckParameterRulespecWithItem,
    rulespec_registry,
    RulespecGroupCheckParametersNetworking,
)


def _parameter_valuespec_checkpoint_fw_ls():
    return Dictionary(elements=[
        ('exp_connection_status',
         DropdownChoice(
             title=_('Expected connection status for the log server'),
             choices=[
                 ('Backup Log-Server Not Active', _('Backup: The log server is a backup log server')),
                 ('Log-Server Connected', _('Connected: The gateway is connected to the log server')),
                 ('Log-Server Disconnected', _('Disconnected: The gateway is not connected to the log server')),
             ],
             default_value='Log-Server Connected',
             )),
        ('mon_connection_state',
         MonitoringState(
             default_value=1,
             title=_('State if connection state differs from expected state.'),
         )),
    ])


rulespec_registry.register(
    CheckParameterRulespecWithItem(
        check_group_name='checkpoint_fw_ls',
        group=RulespecGroupCheckParametersNetworking,
        item_spec=lambda: TextAscii(title=_('Log server IP'), ),
        match_type='dict',
        parameter_valuespec=_parameter_valuespec_checkpoint_fw_ls,
        title=lambda: _('Check Point Firewall gateway log server'),
    ))
